/*  Copyright (c) 2005 Romain BONDUE
    This file is part of RutilT.

    RutilT is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    RutilT is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with RutilT; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/
/** \file RTModules.cxx
    \author Romain BONDUE
    \date 21/08/2005 */
#include <sstream>

#include "RTModules.h"
#include "GtkGUI.h"
#include "RTDrivers.h"
#include "RTDriversRoot.h"
#include "ErrorsCode.h"



namespace
{
    ::GtkWidget* LabelWidget (::GtkWidget* pWidget, const char* Label) throw()
    {
        ::GtkBox* const pHBox (GTK_BOX (::gtk_hbox_new (false, 2)));
        ::gtk_box_pack_start_defaults (pHBox, ::gtk_label_new (Label));
        ::gtk_box_pack_start_defaults (pHBox, pWidget);

        return GTK_WIDGET (pHBox);
        
    } // LabelWidget()


    const unsigned TabAuthRT2400Size (2);
    const nsWireless::AuthType_e TabAuthRT2400 [TabAuthRT2400Size] =
                                        {nsWireless::Open, nsWireless::Shared};

    const unsigned TabAuthRT25x0Size (3);
    const nsWireless::AuthType_e TabAuthRT25x0 [TabAuthRT25x0Size] =
                    {nsWireless::Open, nsWireless::Shared, nsWireless::WPAPSK};

    const unsigned TabAuthAdHocRT25x0Size (3);
    const nsWireless::AuthType_e TabAuthAdHocRT25x0 [TabAuthAdHocRT25x0Size] =
                {nsWireless::Open, nsWireless::Shared, nsWireless::WPANONE};

    const unsigned TabNoneEncryptSize (2);
    const nsWireless::EncryptType_e TabNoneEncrypt [TabNoneEncryptSize] =
                                        {nsWireless::None, nsWireless::WEP};

    const unsigned TabSharedEncryptSize (1);
    const nsWireless::EncryptType_e TabSharedEncrypt [TabSharedEncryptSize] =
                                                            {nsWireless::WEP};

    const unsigned TabWPAPSKRTxxEncryptSize (2);
    const nsWireless::EncryptType_e TabWPAPSKRTxxEncrypt
            [TabWPAPSKRTxxEncryptSize] = {nsWireless::TKIP, nsWireless::AES};

    const unsigned TabAuthRT61Size (4);
    const nsWireless::AuthType_e TabAuthRT61 [TabAuthRT61Size] =
                                    {nsWireless::Open, nsWireless::Shared,
                                     nsWireless::WPAPSK, nsWireless::WPA2PSK};

        // FIXME Are they all really supported by rt2400 ?
    const unsigned TabModeRT2xx0Size (3);
    const nsWireless::Mode_e TabModeRT2xx0 [TabModeRT2xx0Size] =
                {nsWireless::Managed, nsWireless::AdHoc, nsWireless::Monitor};


    void ApplyRT2400Profile (const nsUserData::CProfile& Profile,
                             nsWireless::CRT2400Driver& Driver)
                                                    throw (nsErrors::CSystemExc)
    {
        Driver.SetChannel (Profile.GetChannel());
        Driver.SetMode (Profile.GetMode());
        Driver.SetEncryption (Profile.GetEncryptionD());
        Driver.SetSSID (Profile.GetSSID());  

    } // ApplyRT2400Profile()


    void ApplyRTxxProfile (const nsUserData::CProfile& Profile,
                             nsWireless::IDriver& Driver)
                                                    throw (nsErrors::CSystemExc)
    {
        Driver.SetChannel (Profile.GetChannel());
        Driver.SetMode (Profile.GetMode());
        Driver.SetSSID (Profile.GetSSID());  
        Driver.SetEncryption (Profile.GetEncryptionD());

    } // ApplyRTxxProfile()

} // anonymous namespace



const char* const nsCore::CRT2400Module::NewProfileDefaultName ("New profile");


const char* const nsCore::CRTAdvancedModule::NewProfileDefaultName
                                                                ("New profile");


const char* const nsCore::CRT2570Module::NewProfileDefaultName ("New profile");



void nsCore::CRT2400CLIModule::ApplyProfile (unsigned Pos)
                                                    throw (nsErrors::CSystemExc)
{
    ApplyRT2400Profile (m_ProfilesFile [Pos], m_Driver);

} // ApplyProfile()


nsGUI::CRT25x0ConfigDialog::CRT25x0ConfigDialog
                            (const nsWireless::EncryptType_e* TabWPAEncrypt,
                             unsigned TabWPAEncryptSize) throw()
    : CSharedConfigDialog (TabModeRT2xx0, TabModeRT2xx0Size,
                           TabAuthRT25x0, TabAuthRT25x0Size,
                           TabNoneEncrypt, TabNoneEncryptSize,
                           TabSharedEncrypt, TabSharedEncryptSize,
                           TabWPAEncrypt, TabWPAEncryptSize)
{
    ::g_signal_connect (G_OBJECT (GetModeBox().operator ::GtkWidget*()),
                        "changed", G_CALLBACK (ModeChangedCallBack), this);

} // CRT25x0ConfigDialog


void nsGUI::CRT25x0ConfigDialog::ModeChangedCallBack (::GtkToggleButton*,
                                          CRT25x0ConfigDialog* This) throw()
{
    if (This->GetModeBox().GetActive() == nsWireless::AdHoc)
    {
        This->GetAuthBox().SetElem (TabAuthAdHocRT25x0,
                                    TabAuthAdHocRT25x0Size);
        This->GetAuthBox().SetActive (TabAuthAdHocRT25x0 [0]);
    }
    else
    {
        This->GetAuthBox().SetElem (TabAuthRT25x0, TabAuthRT25x0Size);
        This->GetAuthBox().SetActive (TabAuthRT25x0 [0]);
    }

} // ModeChangedCallBack()


nsGUI::CRT61ConfigDialog::CRT61ConfigDialog () throw()
    : CSharedConfigDialog (TabModeRT2xx0, TabModeRT2xx0Size,
                           TabAuthRT61, TabAuthRT61Size,
                           TabNoneEncrypt, TabNoneEncryptSize,
                           TabSharedEncrypt, TabSharedEncryptSize,
                           TabWPAPSKRTxxEncrypt, TabWPAPSKRTxxEncryptSize)
{
    ::g_signal_connect (G_OBJECT (GetModeBox().operator ::GtkWidget*()),
                        "changed", G_CALLBACK (ModeChangedCallBack), this);

} // CRT61ConfigDialog


void nsGUI::CRT61ConfigDialog::ModeChangedCallBack (::GtkToggleButton*,
                                              CRT61ConfigDialog* This) throw()
{
    if (This->GetModeBox().GetActive() == nsWireless::AdHoc)
    {
        This->GetAuthBox().SetElem (TabAuthAdHocRT25x0,
                                    TabAuthAdHocRT25x0Size);
        This->GetAuthBox().SetActive (TabAuthAdHocRT25x0 [0]);
    }
    else
    {
        This->GetAuthBox().SetElem (TabAuthRT61, TabAuthRT61Size);
        This->GetAuthBox().SetActive (TabAuthRT61 [0]);
    }

} // ModeChangedCallBack()


nsCore::CRT2400Module::CRT2400Module (const std::string& DeviceName,
                                      const std::string& Path)
                                 throw (nsErrors::CException, std::bad_alloc)
    : m_pDriver (new nsWireless::CRT2400Driver (DeviceName)),
      m_ProfileEditor (new nsGUI::CSharedConfigDialog
                                      (TabModeRT2xx0, TabModeRT2xx0Size,
                                       m_pDriver->GetSupportedFreq(),
                                       TabAuthRT2400, TabAuthRT2400Size,
                                       TabNoneEncrypt, TabNoneEncryptSize,
                                       TabSharedEncrypt, TabSharedEncryptSize,
                                       0, 0)),
      m_ProfilesFile (Path)
{
    m_ProfilesFile.Extract();

} // CRT2400Module()


void nsCore::CRT2400Module::BecomeRoot () throw (nsErrors::CException,
                                                 std::bad_alloc)
{
    m_pDriver.reset (new nsWireless::CRT2400DriverRoot
                                                (m_pDriver->GetDeviceName()));

} // BecomeRoot()


void nsCore::CRT2400Module::RecordProfiles () throw (nsErrors::CException)
{
    m_ProfilesFile.Record();

} // RecordProfiles()


void nsCore::CRT2400Module::Connect (const nsWireless::CCell& Cell)
                                   throw (nsErrors::CSystemExc, std::bad_alloc)
{
    if (Cell.GetMode() == nsWireless::AdHoc);
    {
        if (Cell.GetChannel())
            m_pDriver->SetChannel (Cell.GetChannel());
        else throw nsErrors::CSystemExc ("Can't connect to an ad-hoc network"
                                         " this way, please use a profile.",
                                         nsErrors::RT2400AdHocLimitation);
    }
    m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
    m_pDriver->SetSSID (Cell.GetSSID());
    m_pDriver->SetEncryption (Cell.GetEncryptionD());

} // Connect()


void nsCore::CRT2400Module::ApplyProfile (unsigned Pos)
                                                throw (nsErrors::CSystemExc)
{
    ApplyRT2400Profile (m_ProfilesFile [Pos], *m_pDriver);

} // ApplyProfile()


nsUserData::CProfile* nsCore::CRT2400Module::EditNewProfile () throw()
{
    if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
    {
        m_ProfilesFile.DeleteLastProfile();
        return 0;
    }
    return &m_ProfilesFile.back();

} // EditNewProfile()


void nsCore::CRTAdvancedCLIModule::ApplyProfile (unsigned Pos)
                                                    throw (nsErrors::CSystemExc)
{
    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);

} // ApplyProfile()


nsCore::CRTAdvancedModule::CRTAdvancedModule
        (nsWireless::CRTAdvancedDriver* pDriver, nsGUI::CConfigDialog* pDialog,
         const std::string& Path, nsGUI::CGtkGUI* pUI)
                                  throw (nsErrors::CException, std::bad_alloc)
    : m_pDriver (pDriver), m_ProfileEditor (pDialog), m_ProfilesFile (Path),
      m_pTable (GTK_TABLE (::gtk_table_new (5, 2, true))), m_pUI (pUI),
      m_CurrentActiveBoxItem (0), m_IsSkipped (false)
{
    m_ProfilesFile.Extract();
        // Build the private tab :
    ::gtk_container_set_border_width (GTK_CONTAINER (m_pTable), 6);
    ::gtk_table_set_col_spacings (m_pTable, 12);
    ::gtk_table_set_row_spacings (m_pTable, 8);

    const unsigned NbMode (2);
    const char* ModeNames [NbMode] = {"Mixed", "B Only"};
    CreateCBox (ModeNames, NbMode, m_pDriver->GetWirelessMode(),
                "Wireless Mode", 0, &ModeCallBack);

    const unsigned NbProtectionMode (3);
    const char* ProtectionModeNames [NbProtectionMode] = {"Auto", "On", "Off"};
    CreateCBox (ProtectionModeNames, NbProtectionMode,
                m_pDriver->GetBGProtection(), "B/G Protection", 1,
                &BGProtectionCallBack);

    const unsigned NbTxPreamble (3);
    const char* TxPreambleNames [NbTxPreamble] = {"Long", "Short", "Auto"};
    CreateCBox (TxPreambleNames, NbTxPreamble, m_pDriver->GetTxPreamble(),
                "Tx Preamble", 2, &TxPreambleCallBack);

    CreateCheckButton ("Tx burst", 3, 0, m_pDriver->GetTxBurst(),
                       &TxBurstCallBack);
    CreateCheckButton ("Turbo rate", 3, 1, m_pDriver->GetTurboRate(),
                                 &TurboRateCallBack);
    CreateCheckButton ("OFDM in ad-hoc mode", 4, 0,
                       m_pDriver->GetAdHocOFDM(), &AdHocOFDMCallBack);

    CreateCheckButton ("Injection in monitor mode", 4, 1,
                       m_pDriver->GetRfmontx(), &RfmontxCallBack);

} // CRTAdvancedModule()


void nsCore::CRTAdvancedModule::RecordProfiles () throw (nsErrors::CException)
{
    m_ProfilesFile.Record();

} // RecordProfiles()


void nsCore::CRTAdvancedModule::CreateCBox
        (const char* TabName [], unsigned TabSize, int CurrentActive,
         const char* Label, unsigned Row,
         void (*CallBack) (::GtkComboBox*, CRTAdvancedModule*)) throw()
{

    ::GtkComboBox* const pCBox (GTK_COMBO_BOX (::gtk_combo_box_new_text()));
    for (unsigned i (0) ; i < TabSize ; ++i)
        ::gtk_combo_box_append_text (pCBox, TabName [i]);
    ::gtk_combo_box_set_active (pCBox, CurrentActive);
    ::g_signal_connect (G_OBJECT (pCBox), "changed", G_CALLBACK (CallBack),
                        this);
    ::g_signal_connect (G_OBJECT (pCBox), "set-focus-child",
                        G_CALLBACK (CBoxFocusCallBack), this);
    ::GtkWidget* const pLabel (::gtk_label_new (Label));
    ::gtk_table_attach_defaults (m_pTable, pLabel, 0, 1, Row, Row + 1);
    ::gtk_table_attach (m_pTable, GTK_WIDGET (pCBox), 1, 2, Row, Row + 1,
                        GTK_FILL, GTK_SHRINK, 0, 0);

} // CreateCBox()


void nsCore::CRTAdvancedModule::CreateCheckButton (
         const char* Label, unsigned Row, unsigned Column, bool CurrentState,
         void (*CallBack) (::GtkToggleButton*, CRTAdvancedModule*)) throw()
{
    ::GtkWidget* const pButton (::gtk_check_button_new_with_label (Label));
    ::gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pButton), CurrentState);
    ::g_signal_connect (G_OBJECT (pButton), "toggled", G_CALLBACK (CallBack),
                        this);
    ::gtk_table_attach_defaults (m_pTable, pButton, Column, Column + 1, Row,
                                 Row + 1);

} // CreateCheckButton()


nsUserData::CProfile* nsCore::CRTAdvancedModule::EditNewProfile () throw()
{
    if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
    {
        m_ProfilesFile.DeleteLastProfile();
        return 0;
    }
    return &m_ProfilesFile.back();

} // EditNewProfile()


void nsCore::CRTAdvancedModule::Connect (const nsWireless::CCell& Cell)
                                throw (nsErrors::CSystemExc, std::bad_alloc)
{
    m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
    m_pDriver->SetChannel (Cell.GetChannel());
    m_pDriver->SetSSID (Cell.GetSSID());
    try
    {
        m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
    }
    catch (...) {} // It's not useful anyway.
    m_pDriver->SetEncryption (Cell.GetEncryptionD());

} // Connect()


void nsCore::CRTAdvancedModule::ApplyProfile (unsigned Pos)
                                                throw (nsErrors::CSystemExc)
{
    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);

} // ApplyProfile()


void nsCore::CRTAdvancedModule::ModeCallBack (::GtkComboBox* pCBox,
                                             CRTAdvancedModule* This) throw()
{
    This->CommonCBoxCallBack (reinterpret_cast<DriverCBoxMember_t>
                            (&nsWireless::CRTAdvancedDriver::SetWirelessMode),
                              pCBox);
    This->m_pUI->RefreshRates();

} // ModeCallBack()


void nsCore::CRT2500Module::BecomeRoot () throw (nsErrors::CException,
                                                 std::bad_alloc)
{
    SetDriver (new nsWireless::CRT2500DriverRoot (GetDriver().GetDeviceName()));

} // BecomeRoot()


void nsCore::CRT61Module::BecomeRoot () throw (nsErrors::CException,
                                               std::bad_alloc)
{
    SetDriver (new nsWireless::CRT61DriverRoot (GetDriver().GetDeviceName()));

} // BecomeRoot()


void nsCore::CRTAdvancedModule::CommonCBoxCallBack (DriverCBoxMember_t Member,
                                                ::GtkComboBox* pCBox) throw()
{
    try
    {
        if (!m_IsSkipped)
        {
            if (m_pUI->BecomeRoot())
            {
                const unsigned CurrentActive
                                        (::gtk_combo_box_get_active (pCBox));
                (m_pDriver.get()->*Member) (CurrentActive);
                m_CurrentActiveBoxItem = CurrentActive;
            }
            else ResumePreviousState (pCBox);
        }
    }
    catch (const nsErrors::CException& Exc)
    {
        ResumePreviousState (pCBox);
        m_pUI->ProcessError (Exc);
    }

} // CommonCBoxCallBack()


void nsCore::CRTAdvancedModule::CommonCButtonCallBack
            (DriverButtonMember_t Member, ::GtkToggleButton* pButton) throw()
{
    try
    {
        const bool NewState (::gtk_toggle_button_get_active (pButton));
        if (!m_IsSkipped)
        {
            if (m_pUI->BecomeRoot())
                (m_pDriver.get()->*Member) (NewState);
            else
            {
                m_IsSkipped = true;
                ::gtk_toggle_button_set_active (pButton, !NewState);
                m_IsSkipped = false;
            }
        }
    }
    catch (const nsErrors::CException& Exc)
    {
        m_pUI->ProcessError (Exc);
    }

} // CommonCButtonCallBack()


void nsCore::CRT2570CLIModule::ApplyProfile (unsigned Pos)
                                                    throw (nsErrors::CSystemExc)
{
    ApplyRTxxProfile (m_ProfilesFile [Pos], m_Driver);

} // ApplyProfile()


nsCore::CRT2570Module::CRT2570Module (const std::string& DeviceName,
                                      const std::string& Path,
                                      nsGUI::CGtkGUI* pUI)
                                throw (nsErrors::CException, std::bad_alloc)
    : m_pDriver (new nsWireless::CRT2570Driver (DeviceName)),
      m_ProfileEditor (new nsGUI::CRT25x0ConfigDialog
                                                    // FIXME -1 to disable AES.
                          (TabWPAPSKRTxxEncrypt, TabWPAPSKRTxxEncryptSize - 1)),
      m_ProfilesFile (Path), m_pUI (pUI),
      m_pBox (GTK_BOX (::gtk_vbox_new (true, 2))), m_CurrentActiveBoxItem (0),
      m_IsSkipped (false)
{
    m_ProfileEditor.GetDialog()->SetAvailableChannel
                                                (m_pDriver->GetSupportedFreq());
    m_ProfilesFile.Extract();
    BuildPrivateTab();

} // CRT2570Module()


void nsCore::CRT2570Module::RecordProfiles () throw (nsErrors::CException)
{
    m_ProfilesFile.Record();

} // RecordProfiles()


void nsCore::CRT2570Module::BecomeRoot () throw (nsErrors::CException,
                                                 std::bad_alloc)
{
    m_pDriver.reset (new nsWireless::CRT2570DriverRoot
                                                (m_pDriver->GetDeviceName()));

} // BecomeRoot()


void nsCore::CRT2570Module::Connect (const nsWireless::CCell& Cell)
                                throw (nsErrors::CSystemExc, std::bad_alloc)
{
    m_pDriver->SetMode (nsWireless::GetMatchingMode (Cell.GetMode()));
    m_pDriver->SetChannel (Cell.GetChannel());
    m_pDriver->SetSSID (Cell.GetSSID());
    m_pDriver->SetAPMacAddr (Cell.GetAPMacAddress());
    m_pDriver->SetEncryption (Cell.GetEncryptionD());
    // ? FIXME ? Does the SSID need to be set again? (WPA)

} // Connect()


void nsCore::CRT2570Module::ApplyProfile (unsigned Pos)
                                                throw (nsErrors::CSystemExc)
{
    ApplyRTxxProfile (m_ProfilesFile [Pos], *m_pDriver);
    // ? FIXME ? Does the SSID need to be set again? (WPA)

} // ApplyProfile()


nsUserData::CProfile* nsCore::CRT2570Module::EditNewProfile () throw()
{
    if (!m_ProfileEditor.Edit (m_ProfilesFile.back()))
    {
        m_ProfilesFile.DeleteLastProfile();
        return 0;
    }
    return &m_ProfilesFile.back();

} // EditNewProfile()


void nsCore::CRT2570Module::CommonCButtonCallBack
            (DriverButtonMember_t pMember, ::GtkToggleButton* pButton) throw()
{
    try
    {
        const bool NewState (::gtk_toggle_button_get_active (pButton));
        if (!m_IsSkipped)
        {
            if (m_pUI->BecomeRoot())
                (m_pDriver.get()->*pMember) (NewState);
            else
            {
                m_IsSkipped = true;
                ::gtk_toggle_button_set_active (pButton, !NewState);
                m_IsSkipped = false;
            }
        }
    }
    catch (const nsErrors::CException& Exc)
    {
        m_pUI->ProcessError (Exc);
    }

} // CommonCButtonCallBack()


void nsCore::CRT2570Module::CommonCBoxCallBack (DriverBoxMember_t pMember,
                                                ::GtkComboBox* pCBox) throw()
{
    try
    {
        if (!m_IsSkipped)
        {
            if (m_pUI->BecomeRoot())
            {
                const int CurrentActive (::gtk_combo_box_get_active (pCBox));
                (m_pDriver.get()->*pMember) (CurrentActive);
                m_CurrentActiveBoxItem = CurrentActive;
            }
            else ResumePreviousState (pCBox);
        }
    }
    catch (const nsErrors::CException& Exc)
    {
        ResumePreviousState (pCBox);
        m_pUI->ProcessError (Exc);
    }

} // AdhocModeCallBack()


void nsCore::CRT2570Module::BuildPrivateTab () throw()
{
    ::GtkComboBox* pCBox (GTK_COMBO_BOX (::gtk_combo_box_new_text()));
    ::gtk_combo_box_append_text (pCBox, "Standard 11Mpbs");
    ::gtk_combo_box_append_text (pCBox, "Auto");
    ::gtk_combo_box_append_text (pCBox, "54Mbps only");
        // Warning : works because of the numeric values, can easily break.
    ::gtk_combo_box_set_active (pCBox, m_pDriver->GetAdHocMode());
    ::g_signal_connect (G_OBJECT (pCBox), "changed",
                        G_CALLBACK (AdhocModeCallBack), this);
    ::g_signal_connect (G_OBJECT (pCBox), "set-focus-child",
                        G_CALLBACK (CBoxFocusCallBack), this);
    ::GtkBox* pHbox (GTK_BOX (::gtk_hbox_new (true, 2)));
    ::gtk_box_pack_start_defaults (pHbox, ::gtk_label_new ("Ad-hoc mode"));
    ::gtk_box_pack_start_defaults (pHbox, GTK_WIDGET (pCBox));
    ::gtk_box_pack_start_defaults (m_pBox, GTK_WIDGET (pHbox));

    ::GtkWidget* const pButton (::gtk_toggle_button_new_with_label
                                                ("Injection in monitor mode"));
    ::gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (pButton),
                                    m_pDriver->GetRfmontx());
    ::g_signal_connect (G_OBJECT (pButton), "toggled",
                        G_CALLBACK (RfmontxCallBack), this);
    ::gtk_box_pack_start_defaults (m_pBox, pButton);

    pCBox = GTK_COMBO_BOX (::gtk_combo_box_new_text());
    ::gtk_combo_box_append_text (pCBox, "Default (auto)");
    ::gtk_combo_box_append_text (pCBox, "Force");
    ::gtk_combo_box_append_text (pCBox, "Force no");
        // Warning : works because of the numeric values, can easily break.
    ::gtk_combo_box_set_active (pCBox, m_pDriver->GetPrismHeader());
    ::g_signal_connect (G_OBJECT (pCBox), "changed",
                        G_CALLBACK (PrismHeaderCallBack), this);
    ::g_signal_connect (G_OBJECT (pCBox), "set-focus-child",
                        G_CALLBACK (CBoxFocusCallBack), this);
    pHbox = GTK_BOX (::gtk_hbox_new (true, 2));
    ::gtk_box_pack_start_defaults (pHbox, ::gtk_label_new ("Prism header"));
    ::gtk_box_pack_start_defaults (pHbox, GTK_WIDGET (pCBox));
    ::gtk_box_pack_start_defaults (m_pBox, GTK_WIDGET (pHbox));

} // BuildPrivateTab()


nsCore::CRT2500Module::CRT2500Module (const std::string& DeviceName,
                            const std::string& ProfilePath, nsGUI::CGtkGUI* pUI)
                                    throw (nsErrors::CException, std::bad_alloc)
    : CRTAdvancedModule (new nsWireless::CRT2500Driver (DeviceName),
                         new nsGUI::CRT25x0ConfigDialog
                               (TabWPAPSKRTxxEncrypt, TabWPAPSKRTxxEncryptSize),
                         ProfilePath, pUI)
{
    GetProfileEditor().GetDialog()->SetAvailableChannel
                                            (GetDriver().GetSupportedFreq());

} // CRT2500Module()


nsCore::CRT61Module::CRT61Module (const std::string& DeviceName,
                            const std::string& ProfilePath, nsGUI::CGtkGUI* pUI)
                                    throw (nsErrors::CException, std::bad_alloc)
    : CRTAdvancedModule (new nsWireless::CRT61Driver (DeviceName),
                         new nsGUI::CRT61ConfigDialog, ProfilePath, pUI)
{
    GetProfileEditor().GetDialog()->SetAvailableChannel
                                            (GetDriver().GetSupportedFreq());

} // CRT61Module()
